/* gasql_xml.c
 *
 * Copyright (C) 1999 Vivien Malerba
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include "gasql_xml.h"

/* will reallocate a string, the one given as argument will be 
   freed */
gchar *gasql_xml_clean_string(gchar *str)
{
  gint nb_spaces = 0;
  gint size = 0;
  gchar *ptr, *ptr2, *retval;

  if (str) {
    ptr = str;
    while (*ptr != '\0') {
      if ((*ptr == ' ') || (*ptr == '\n') || (*ptr == '\t')) 
	nb_spaces ++;
      size ++;
      ptr++;
    }
    
    if (! nb_spaces)
      return str;

    retval = (gchar *) g_malloc(sizeof(gchar) * (size+1));
    ptr = str;
    ptr2 = retval;
    while (*ptr != '\0') {
      if ((*ptr != ' ') && (*ptr != '\n') && (*ptr != '\t')) {
	*ptr2 = *ptr;
	ptr2++;
      }
      ptr++;
    }
    *ptr2 = '\0';
    g_free(str);
    return retval;
  }
  else
    return NULL;
}

/* will reallocate a string, the one given as argument will be 
   freed */
gchar *gasql_xml_clean_string_ends(gchar *str)
{
  gint size;
  gchar *ptr, *ptr2, *retval;
  gchar *lptr = NULL; /* points to the first caracter from the beginning 
			 which is not ' ', '\t' or '\n' */
  gchar *rptr = NULL; /* points to the last caracter from the beginning 
			 which is not ' ', '\t' or '\n' */
  
  if (str) {
    ptr = str;
    while (*ptr != '\0') {
      if (!lptr && (*ptr != ' ') && (*ptr != '\n') && (*ptr != '\t'))
	lptr = ptr; /* left */
      if ((*ptr != ' ') && (*ptr != '\n') && (*ptr != '\t')) 
	rptr = ptr; /* right */
      size ++;
      ptr++;
    }
    
    if (!lptr) { /* only spaces!!! */
      g_free(str);
      return NULL;
    }

    size = (rptr - lptr)+1; 

    retval = (gchar *) g_malloc(sizeof(gchar) * (size+1));
    ptr = lptr;
    ptr2 = retval;
    while ((ptr != rptr+1)) {
      *ptr2 = *ptr;
      ptr2++;
      ptr++;
    }
    *ptr2 = '\0';
    g_free(str);
    return retval;
  }
  else
    return NULL;
}


/* is called to replace some references */
GSList * gasql_xml_update_ref1(GSList *refs, gpointer param, 
			       gasqlXmlUpdateRef1 func)
{
  GSList *list = refs, *returned = refs, *hold;
  gasqlXmlNodeRef *ref;
  gpointer found;

  while (list) {
    ref = (gasqlXmlNodeRef *)(list->data);
    /*g_print("XMLReplace: %s...", ref->txt);*/
    if ((found = (*func)(param, ref->txt))) {
      *(ref->to_replace) = found;
      hold = g_slist_next(list);
      returned = gasql_xml_remove_ref_list(returned, list);
      list = hold;
    }
    else 
      list = g_slist_next(list);
  }
  return returned;
}

GSList * gasql_xml_update_ref2(GSList *refs, gpointer param, 
			       gasqlXmlUpdateRef2 func)
{
  GSList *list = refs, *returned = refs, *hold;
  gasqlXmlNodeRef *ref;
  gpointer found;

  while (list) {
    ref = (gasqlXmlNodeRef *)(list->data);
    if ((found = (*func)(param, ref->txt, ref->txt2))) {
      *(ref->to_replace) = found;
      hold = g_slist_next(list);
      returned = gasql_xml_remove_ref_list(returned, list);
      list = hold;
    }
    else 
      list = g_slist_next(list);
  }
  return returned;
}

/* WARNING: txt and txt2 need to be an allocated string and MUST NOT be freed 
   (or must be NULL if only one param is used)!
   give NULL as the list argument for the beginning of the list */
GSList *gasql_xml_append_ref_list(GSList *list, 
				  gchar *txt, gchar *txt2,
				  gpointer *to_replace)
{
  GSList *ret;
  gasqlXmlNodeRef *ref;

  ref = (gasqlXmlNodeRef *) g_malloc(sizeof(gasqlXmlNodeRef));
  ref->txt = txt;
  ref->txt2 = txt2;
  ref->to_replace = to_replace;
  ret = g_slist_append(list, ref);
  return ret;
}

/* takes care of freeing any text allocated and not used anymore */
GSList *gasql_xml_remove_ref_list(GSList *list, GSList *node)
{
  gasqlXmlNodeRef *ref;
  GSList *ret;

  ref = (gasqlXmlNodeRef *)(node->data);
  if (ref) {
    g_free(ref->txt);
    if (ref->txt2) 
      g_free(ref->txt2);
    g_free(ref);
  }
  ret = g_slist_remove_link(list, node);
  return ret;
}
