/* interface_cb.c
 *
 * Copyright (C) 1999,2000 Vivien Malerba
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include "interface_cb.h"
#include "sqlwiddbtree.h"
#include "sqlwiddbrel.h"
#include "sqldatawid.h"
#include "mainpagequery.h"
#include <gnome.h>
#include <gda/gda-ui.h>
#ifdef HAVE_BONOBO
#include <bonobo.h>
#include <gnome-db-control-widget.h>
#endif

static void show_relations_dialog_clicked_cb(GnomeDialog * dialog, 
					     gint button_number, 
					     gpointer data);
static void show_relations_dialog_close_cb(GtkObject *obj, 
					   GnomeDialog *dialog);



void ask_to_refresh_after_connect_cb(gint reply, gASQL_Main_Config *conf)
{
  if (reply == 0) 
    sql_mem_update_cb(NULL, conf);
}


/* CB from the server to tell the connection is OK */
void sql_server_conn_open_cb(GtkObject *wid, gpointer data)
{
  gASQL_Main_Config *conf;
  GSList *list, *table_refs, *seq_refs;
  xmlDocPtr doc;
  xmlNodePtr tree, subtree, subsubtree;
  gboolean ok, csud;
  gchar *txt, *txt2;
  WidRelData *reldata;
  SqlQuery *q;

  conf = (gASQL_Main_Config *) data;
  gnome_app_flash(GNOME_APP(((gASQL_Main_Config *)data)->app),
		  _("Connection to the SQL server opened"));
  csud = conf->save_up_to_date;

  /* try to load the structure from an XML file if it exists */
  if (conf->working_file && g_file_test(conf->working_file, 
					G_FILE_TEST_ISFILE)) 
    {
      /* 
       * load the XML file 
       */
      ok = TRUE;
      doc = xmlParseFile(conf->working_file);
      if (doc) 
	{
	  tree = doc->root->childs;
	  while (tree) 
	    {
	      txt = gasql_xml_clean_string(xmlNodeGetContent(tree));
	      /*
	       * SqlAccess object
	       */
	      if (!strcmp(tree->name, "SQLACCESS")) 
		ok = ok | sql_access_build_from_xml_tree(conf->srv, tree);
	      
	      /*
	       * SqlDb object
	       */
	      if (!strcmp(tree->name, "SQLDB")) 
		ok = ok | sql_db_build_db_from_xml_tree(conf->db, tree);
	      
	      /*
	       * SqlWidDbRel object 
	       */
	      if (!strcmp(tree->name, "SQLWIDDBREL")) 
		{ 
		  table_refs = NULL;
		  seq_refs = NULL;
		  /* loading all the 'dbrel' tags */
		  subtree = tree->childs;
		  while (subtree) 
		    {
		      txt2 = gasql_xml_clean_string
			(xmlNodeGetContent(subtree));
		      reldata = (WidRelData *) 
			g_malloc(sizeof(WidRelData));
		      reldata->name = txt2;
		      txt2 = xmlGetProp(subtree, "width");
		      if (txt2)
			reldata->size.width= atoi(txt2);
		      else
			reldata->size.width = 300;
		      txt2 = xmlGetProp(subtree, "height");
		      if (txt2)
			reldata->size.height= atoi(txt2);
		      else
			reldata->size.height = 200;
		      reldata->nodes = NULL;
		      subsubtree = subtree->childs;
		      /* loading all the 'node' tags */
		      while (subsubtree) 
			{
			  reldata->nodes=
			    sql_wid_db_rel_build_from_xml_tree(reldata->nodes,
							       &table_refs,
							       &seq_refs,
							       subsubtree);
			  subsubtree = subsubtree->next;
			}
		      /* updating links */
		      table_refs = gasql_xml_update_ref1(table_refs, conf->db, 
			     GASQL_XML_UPDATE_REF1(sql_db_find_table_by_name));
		      seq_refs = gasql_xml_update_ref1(seq_refs, conf->db, 
			  GASQL_XML_UPDATE_REF1(sql_db_find_sequence_by_name));
		      subtree = subtree->next;
		      if (seq_refs || table_refs) 
			{
			  /* remove broken nodes */
			  list = reldata->nodes;
			  while(list) {
			    if (! ((WidRelNode *)(list->data))->object) 
			      {
				g_free (list->data);
				reldata->nodes = 
				  g_slist_remove_link(reldata->nodes, list);
				list = reldata->nodes;
			      }
			    else
			      list = g_slist_next(list);
			  }
			  /* empty seq_refs && table_refs */
			  while (seq_refs)
			    seq_refs = gasql_xml_remove_ref_list(seq_refs, 
								 seq_refs);
			  while (table_refs)
			    table_refs = gasql_xml_remove_ref_list(table_refs,
								   table_refs);
			}
		      conf->widdbrel_list = g_slist_append(conf->widdbrel_list,
							   reldata);
		    }
		}
	      

	      /*
	       * SqlQuery Object
	       */
	      if (!strcmp(tree->name, "QUERY")) 
		{ 
		  /* loading one query at a time */
		  q = sql_query_build_from_xml_tree(conf, tree);
		  /* if the query was correctly loaded, add it to the 
		     MainPageQuery */
		  if (q)
		    main_page_query_add_query
		      (MAIN_PAGE_QUERY(conf->queries_page), q);
		  ok = (q)? ok : FALSE;
		}
	      tree = tree->next;
	      if (txt) 
		g_free(txt);
	    }
	  xmlFreeDoc (doc);
	  conf->save_up_to_date = csud;
	}
      else
	ok = FALSE;

      if (!ok)
	gnome_warning_dialog(_("An error has occured while loading the "
			       "selected document. The file (XML format)\n"
			       "is probably corrupted. Try to repair it."));
    
      /* ask to refresh the structure */
      txt = g_strdup_printf(_("DataBase '%s':\n"
			      "do you want to update the list of\n data types,"
			      " tables, functions, "
			      "sequences, ... from the server\n(recommanded "
			      "if the structure of the DB has been "
			      "modified).\n"
			      "Say Yes if unsure."),
			    conf->srv->gda_datasource->str);
      gnome_question_dialog(txt,
			  (GnomeReplyCallback) ask_to_refresh_after_connect_cb,
			    conf);
      g_free(txt);
    }
  else 
    {
      sql_access_refresh_datas(conf->srv);
      sql_db_refresh(conf->db, conf->srv);  
    }

  /* sets the notebook to be displayed */
  if (conf->srv->features.sequences)
    gtk_widget_show(conf->sequences_page);
  else
    gtk_widget_hide(conf->sequences_page);
  gtk_widget_show (conf->nb);
  gtk_widget_hide (conf->welcome);
}



static void dummy_cb(gint reply, gpointer data)
{
}
/* CB from the server to tell the connection is going DOWN */
void sql_server_conn_to_close_cb(GtkObject *wid, gpointer data)
{
  gASQL_Main_Config *conf;
  conf = (gASQL_Main_Config *) data;

  if (conf->working_file && !conf->save_up_to_date) {
    GtkWidget *wid;

    wid = gnome_app_question_modal(GNOME_APP(conf->app), 
				   _("Some work has not been "
				     "saved.\nDo you want to save it now?"), 
				   (GnomeReplyCallback)(dummy_cb), conf);
    switch (gnome_dialog_run_and_close(GNOME_DIALOG(wid))) {
    case 0: /* save */
	file_save_cb (NULL, conf);
      break;
    default: /* do nothing */
      break;
    }
  }
  if (conf->check_dlg) {
    conf->check_perform = FALSE;
    gnome_dialog_close(GNOME_DIALOG(conf->check_dlg));
    conf->check_dlg = NULL;
    conf->check_pbar = NULL;
    conf->check_link_name = NULL;
    conf->check_errors = NULL;
  }
}


/* CB from the server to tell the connection is DOWN */
void sql_server_conn_close_cb(GtkObject *wid, gpointer data)
{
  gASQL_Main_Config *conf;
  GSList *list, *list2;
  
  conf = (gASQL_Main_Config *) data;

  gnome_app_flash(GNOME_APP(((gASQL_Main_Config *)data)->app),
		  _("Connection to the SQL server closed"));

  /* remove all the data for any SqlWidDbRel object */
  list = conf->widdbrel_list;
  while(list) {
    list2 = ((WidRelData *)(list->data))->nodes;
    while(list2) {
      g_free(list2->data);
      list2 = g_slist_remove_link(list2, list2);
    }
    g_free(((WidRelData *)(list->data))->name);
    g_free(list->data);
    list = g_slist_remove_link(list, list);
  }
  conf->widdbrel_list = NULL;

  /* Removing all the queries in config */
  list = conf->queries;
  while(list) {
    gtk_object_destroy(GTK_OBJECT(list->data));
    list = g_slist_remove_link(list, list);
  }
  conf->queries = NULL;

  /* sets the welcome message to be displayed */
  gtk_widget_hide (conf->nb);
  gtk_widget_show (conf->welcome);
}


/* Cb to open the connexion to the SQL server */
void sql_conn_open_cb (GtkWidget *widget, void *data)
{
  gASQL_Main_Config *conf;
  conf = (gASQL_Main_Config *) data;
  if (*(conf->srv->gda_datasource->str) != '\0') 
    {
      if (conf->config_dlg)
	gtk_object_destroy(GTK_OBJECT(conf->config_dlg));
      sql_access_open_connect(conf->srv);
    }
  else 
    {
      options_config_cb (NULL, data);
      /* sets the "Apply" button active because we consider a modif done */
      gnome_property_box_changed(GNOME_PROPERTY_BOX(conf->config_dlg));
    }
}

/* Cb to close the connexion for the SQL server */
void sql_conn_close_cb (GtkWidget *widget, void *data)
{
  sql_access_close_connect(((gASQL_Main_Config *)(data))->srv);
}


/* CB to coordinate SqlAccess and SqlDb if an error occurs in SqlDb ("fault" 
   signal). data points on a gASQL_Main_Config struct */
void access_db_link_cb(GtkObject *obj, gpointer data)
{
  gnome_app_message(GNOME_APP(((gASQL_Main_Config *)data)->app),
		    _("A fault in the memory's representation of the database "
		    "structure is driving at disconnecting from the SQL "
		    "server."));
  sql_access_close_connect(SQL_ACCESS(((gASQL_Main_Config *)data)->srv));  
}

/* Cb to catch the SQL server errors. */
static gint err_dlg_closed(GnomeDialog * dialog, gASQL_Main_Config *conf)
{
  conf->error_dlg = NULL;
  return 0;
}
void sql_server_catch_errors_cb(GtkWidget *wid, GList *list, 
				gASQL_Main_Config *conf)
{
  if (!conf->error_dlg)
    {
      conf->error_dlg = gnome_db_errordlg_new(GDA_CONNECTION(conf->srv), 
					      _("Error Viewer"));
      gnome_db_errordlg_show_errors(GNOME_DB_ERRORDLG(conf->error_dlg)); 
      gtk_signal_connect(GTK_OBJECT(conf->error_dlg), "close",
			 GTK_SIGNAL_FUNC(err_dlg_closed), conf);
    }
}

/* data must point on a gASQL_Main_Config struct */
void sql_data_view_cb(GtkWidget *widget, void *data)
{
  gASQL_Main_Config *conf;
  GtkWidget *dlg;

  conf = (gASQL_Main_Config *) data;
  dlg = sql_data_wid_new_dlg(conf->srv);
  gtk_widget_show(dlg);
}

/* data must point on a gASQL_Main_Config struct */
void sql_mem_update_cb(GtkWidget *widget, void *data)
{
  gASQL_Main_Config *conf;

  conf = (gASQL_Main_Config *) data;

  /* update of the data types known by postgreSQL */
  sql_access_refresh_datas(conf->srv);
  /* let's refresh the memory's representation */
  sql_db_refresh(conf->db, conf->srv);  
}


void sql_show_relations_cb(GtkObject *obj, gpointer data)
{
  static GtkWidget *dlg=NULL;
  gASQL_Main_Config *conf;
  GtkWidget *tree, *paned;
  GtkWidget *mainw;

  /* to hide the DLG */
  if (! data) {
    dlg = NULL;
    return;
  }

  if (!dlg) {
    conf = (gASQL_Main_Config *) data;
    dlg = gnome_dialog_new(_("Database Relations"),
			   GNOME_STOCK_BUTTON_OK, NULL);
    gtk_window_set_policy(GTK_WINDOW(dlg), TRUE, TRUE, FALSE);
    /* paned */
    paned = gtk_hpaned_new();
    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dlg)->vbox), paned,
		       TRUE, TRUE, GNOME_PAD);
    gtk_widget_show(paned);
    
    /* SqlTree */
    tree = sql_wid_db_tree_new(conf->db);
    sql_wid_db_tree_set_mode(SQL_WID_DB_TREE(tree), 
			     SQL_WID_DB_TREE_TABLES | 
			     SQL_WID_DB_TREE_SEQS |
			     SQL_WID_DB_TREE_TABLES_SEL | 
			     SQL_WID_DB_TREE_SEQS_SEL);
    gtk_paned_add1(GTK_PANED(paned), tree);
    gtk_widget_show(tree);

    /* Relations graph */
    mainw = sql_wid_db_rel_new(conf, "main");
    gtk_paned_add2(GTK_PANED(paned), mainw);
    gtk_widget_show(mainw);
    gtk_widget_show(dlg);

    /* signals */
    gtk_signal_connect(GTK_OBJECT(tree), "seq_selected",
		       GTK_SIGNAL_FUNC(sql_wid_db_rel_toggle_seq_cb), mainw);

    gtk_signal_connect(GTK_OBJECT(tree), "table_selected",
		       GTK_SIGNAL_FUNC(sql_wid_db_rel_toggle_table_cb), mainw);

    gtk_signal_connect(GTK_OBJECT(dlg), "clicked",
		       GTK_SIGNAL_FUNC(show_relations_dialog_clicked_cb),
		       NULL);
    gtk_signal_connect_while_alive(GTK_OBJECT(conf->srv), "conn_closed",
				 GTK_SIGNAL_FUNC(show_relations_dialog_close_cb),
			         dlg, GTK_OBJECT(mainw));
  }
  else 
    gdk_window_raise(dlg->window);
}

static void show_relations_dialog_close_cb(GtkObject *obj, GnomeDialog *dialog)
{
  gnome_dialog_close(dialog);
  /* notify the removal of the dialog (by passing NULL) */
  sql_show_relations_cb(NULL, NULL);
}

static void show_relations_dialog_clicked_cb(GnomeDialog * dialog, 
					     gint button_number, 
					     gpointer data)
{
  gnome_dialog_close(dialog);
  /* notify the removal of the dialog (by passing NULL) */
  sql_show_relations_cb(NULL, NULL);
}

/* WARNING: item can be either a SqlMemTable or an SqlMemSequence */
void wid_db_rel_list_update(GtkObject *obj, gpointer item, gpointer data)
{
  gASQL_Main_Config *conf;
  GSList *list, *nodes;
  WidRelData *wdata;
  WidRelNode *node;

  conf = (gASQL_Main_Config *) data;
  list = conf->widdbrel_list;
  while (list) {
    wdata = (WidRelData *)(list->data);
    nodes = wdata->nodes;
    while (nodes) {
      node = (WidRelNode *)(nodes->data);
      if (item == node->object) {
	/* removing that node it */
	wdata->nodes = g_slist_remove_link(wdata->nodes, nodes);
	g_free(node);
	nodes = wdata->nodes;
      }
      else
	nodes = g_slist_next(nodes);
    }
    list = g_slist_next(list);
  }
}

/******************************* Configuration part ************************/

static void apply_sql_server_changes(GnomePropertyBox *props, 
				     gASQL_Main_Config *conf);

static void apply_cb(GnomePropertyBox *obj, gint page, 
		     gASQL_Main_Config *conf)
{
  switch (page) {
  case 0:
    apply_sql_server_changes(obj, conf);
    break;
  default:
    /* nothing to be done */
  }
}

static void apply_sql_server_changes(GnomePropertyBox *props, 
				     gASQL_Main_Config *conf)
{
  gchar *dbclear, *user, *password;
  GnomeDbLogin *login;

  login = GNOME_DB_LOGIN(GTK_WIDGET(
		   gtk_object_get_data(GTK_OBJECT(props), "login")));
  dbclear = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(login->gda_dbname)->entry));
  user = gtk_entry_get_text(GTK_ENTRY(login->username_entry));
  password = gtk_entry_get_text(GTK_ENTRY(login->password_entry));
  conf->save_up_to_date = FALSE;

  
  /* fetching DSN and provider */
  g_string_assign(conf->srv->gda_datasource, dbclear);
  g_string_assign(conf->srv->user_name, user);
  g_string_assign(conf->srv->password, password);
}


static void notify_change_cb(GtkObject *obj, GtkWidget *props)
{
  gnome_property_box_changed(GNOME_PROPERTY_BOX(props));
}

static void notify_destroy_config_dlg_cb(GtkWidget *wid, 
					 gASQL_Main_Config *conf)
{
  conf->config_dlg = NULL;
}

/* CB to configure all the options. Data points to a
   gASQL_Main_Config struct */
void options_config_cb (GtkWidget *widget, void *data)
{
  gASQL_Main_Config *conf;
  GtkWidget *props, *label, *page;

  conf = (gASQL_Main_Config *) data;
  if (conf->config_dlg) {
    gdk_window_raise(conf->config_dlg->window);
    return;
  }

  props = gnome_property_box_new();
  conf->config_dlg = props;

  /* SQL servers config */
  label = gtk_label_new(_("Server configuration"));
  gtk_widget_show(label);
  page = gnome_db_login_new(GDA_CONNECTION(conf->srv), NULL, NULL);
  gtk_object_set_data(GTK_OBJECT(props), "login", page);

  /* Integrate what is already in the variables (gda_dbname,...) */
  if (*conf->srv->gda_datasource->str != '\0')
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(GNOME_DB_LOGIN(page)->gda_dbname)->entry),
		       conf->srv->gda_datasource->str);
  gtk_entry_set_text(GTK_ENTRY(GNOME_DB_LOGIN(page)->username_entry),
		     conf->srv->user_name->str);
  gtk_entry_set_text(GTK_ENTRY(GNOME_DB_LOGIN(page)->password_entry),
		     conf->srv->password->str);

  /* changes notification */
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(GNOME_DB_LOGIN(page)->gda_dbname)->entry), 
		     "changed",
                     GTK_SIGNAL_FUNC(notify_change_cb), props);
  gtk_signal_connect(GTK_OBJECT(GNOME_DB_LOGIN(page)->username_entry), 
		     "changed",
                     GTK_SIGNAL_FUNC(notify_change_cb), props);
  gtk_signal_connect(GTK_OBJECT(GNOME_DB_LOGIN(page)->password_entry), 
		     "changed",
                     GTK_SIGNAL_FUNC(notify_change_cb), props);

  gnome_property_box_append_page(GNOME_PROPERTY_BOX(props), page, label);
  gtk_container_set_border_width(GTK_CONTAINER(page), GNOME_PAD);
  gtk_widget_show(page);

  /* signals */
  gtk_window_set_title(GTK_WINDOW(props), _("Connection preferences"));
  gtk_signal_connect(GTK_OBJECT(props), "apply",
		     GTK_SIGNAL_FUNC(apply_cb), conf);
  gtk_signal_connect(GTK_OBJECT(props), "destroy",
		     GTK_SIGNAL_FUNC(notify_destroy_config_dlg_cb), conf);
  gtk_widget_show(props);

  /* consider that there is necessaryly a change to take into account */
  if (!conf->srv->gda_datasource || 
      (*conf->srv->gda_datasource->str == 0))
    gnome_property_box_changed(GNOME_PROPERTY_BOX(conf->config_dlg));
}


/* some callbacks from the FILE menu */
gint set_opened_file(gASQL_Main_Config *conf, gchar *filetxt)
{
  gchar *txt;
  gint retval =0;
  xmlDocPtr doc;
  xmlNodePtr tree;

  if (!g_file_test(filetxt, G_FILE_TEST_ISFILE)) {
    txt = g_strdup_printf(_("Cannot open file '%s'\n(file does not exist "
			    "or is not readable)."), filetxt);
    gnome_error_dialog(txt);
    g_free(txt);
    return -1;
  }
  else {
    doc = xmlParseFile(filetxt);
    if (doc) {
      txt = xmlGetProp(doc->root, "id_serial");
      if (txt) {
	conf->id_serial = atoi(txt);
      }
      tree = doc->root->childs;
      while (tree) {
	txt = gasql_xml_clean_string_ends(xmlNodeGetContent(tree));
	if (!strcmp(tree->name, "gda_datasource")) {
	  if (txt) 
	    g_string_assign(conf->srv->gda_datasource, txt);
	  else
	    g_string_assign(conf->srv->gda_datasource, "");
	}

	if (!strcmp(tree->name, "username")) {
	  if (txt)
	    g_string_assign(conf->srv->user_name, txt);
	  else
	    g_string_assign(conf->srv->user_name, "");
	}

	tree = tree->next;
	if (txt) 
	  g_free(txt);
      }
      xmlFreeDoc (doc);
      g_string_assign(conf->srv->password, "");

      if (conf->working_file)
	g_free(conf->working_file);
      conf->working_file = filetxt;
      txt = gasql_get_title(conf);
      gtk_window_set_title(GTK_WINDOW(conf->app), txt);
      g_free(txt);
    }
    else {
      gnome_warning_dialog(_("An error has occured while loading the "
			     "connexion parameters.\n"
			     "Set them manually."));
      retval = -1; /* ERROR backpropagation */
    }
  }

  return retval;
}



void file_open_dlg_ok_cb(GtkWidget *widget, void *data)
{
  gASQL_Main_Config *conf;
  gchar *filetxt;
  
  conf = (gASQL_Main_Config *) gtk_object_get_data(GTK_OBJECT(data), "conf");
  filetxt =g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(data)));
  gtk_object_destroy(GTK_OBJECT(data));

  if (set_opened_file(conf, filetxt)==-1)
    g_free(filetxt);  
}

void file_open_dlg_cancel_cb(GtkWidget *widget, void *data)
{
  gtk_object_destroy(GTK_OBJECT(data));
}

void file_open_cb (GtkWidget *widget, void *data)
{
  gASQL_Main_Config *conf;
  GtkWidget *dlg;

  conf = (gASQL_Main_Config *) data;
  /* first test if the connection is established, ... */
  if (sql_access_is_open(conf->srv)) {
    gnome_warning_dialog(_("The connection to an SQL server is already "
			   "established.\nDisconnect before trying to open "
			   "another file."));
    return;
  }

  /* let's now open a dialog... */
  dlg = gtk_file_selection_new(_("File to open"));
  gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(dlg));
  gtk_window_set_modal(GTK_WINDOW(dlg), TRUE);
  gtk_object_set_data(GTK_OBJECT(dlg), "conf", conf);
  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(dlg)->cancel_button), 
		     "clicked", 
		     GTK_SIGNAL_FUNC(file_open_dlg_cancel_cb), dlg);
  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(dlg)->ok_button), 
		     "clicked", 
		     GTK_SIGNAL_FUNC(file_open_dlg_ok_cb), dlg);
  gtk_widget_show(dlg);
}


void file_new_dlg_cancel_cb(GtkWidget *widget, void *data)
{
  gtk_object_destroy(GTK_OBJECT(data));
}

void file_new_dlg_ok_cb(GtkWidget *widget, void *data)
{
  gASQL_Main_Config *conf;
  gchar *filetxt;
  
  conf = (gASQL_Main_Config *) gtk_object_get_data(GTK_OBJECT(data), "conf");
  filetxt =g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(data)));
  gtk_object_destroy(GTK_OBJECT(data));

  if (conf->working_file && !conf->save_up_to_date) {
    GtkWidget *wid;

    wid = gnome_app_question_modal(GNOME_APP(conf->app), 
				   _("Some work has not been "
				     "saved.\nDo you want to save it now?"), 
				   (GnomeReplyCallback)(dummy_cb), conf);
    if (! gnome_dialog_run_and_close(GNOME_DIALOG(wid)))
	file_save_cb (NULL, conf);
  }
  if (conf->working_file)
    g_free(conf->working_file);
  conf->working_file = filetxt;
  filetxt = gasql_get_title(conf);
  gtk_window_set_title(GTK_WINDOW(conf->app), filetxt);
  g_free(filetxt);
}

void file_new_cb (GtkWidget *widget, gASQL_Main_Config *conf)
{
  GtkWidget *dlg;

  dlg = gtk_file_selection_new(_("New file"));
  gtk_window_set_modal(GTK_WINDOW(dlg), TRUE);
  gtk_object_set_data(GTK_OBJECT(dlg), "conf", conf);
  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(dlg)->cancel_button), 
		     "clicked", 
		     GTK_SIGNAL_FUNC(file_new_dlg_cancel_cb), dlg); /* ! */
  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(dlg)->ok_button), 
		     "clicked", 
		     GTK_SIGNAL_FUNC(file_new_dlg_ok_cb), dlg);
  gtk_widget_show(dlg);
}  



/* widget not used! */
void file_save_cb (GtkWidget *widget, void *data)
{
  xmlDocPtr doc;
  xmlNodePtr toptree, tree;
  gASQL_Main_Config *conf;
  GSList *list;
  gchar *str;

  conf = (gASQL_Main_Config *) data;
  if (conf->working_file) {
    doc = xmlNewDoc("1.0");
    doc->root = xmlNewDocNode(doc, NULL, "CONNECTION", NULL);
    str = g_strdup_printf("%d", conf->id_serial);
    xmlSetProp(doc->root, "id_serial", str);
    g_free(str);
    tree = xmlNewChild(doc->root, NULL, "gda_datasource", 
		       conf->srv->gda_datasource->str);
    tree = xmlNewChild(doc->root, NULL, "username", conf->srv->user_name->str);

    /* the SqlAccess object */
    sql_access_build_xml_tree(conf->srv, doc);

    /* the SqlDb object */
    sql_db_build_xml_tree(conf->db, doc);

    /* all the data from SqlWidDbRel objects. it is necessary to go
     manually through all the WidRelData structs at this level, because
     there is not only one object of type SqlWidDbRel can be opened
     per session. */
    toptree = xmlNewChild(doc->root, NULL, "SQLWIDDBREL", NULL);
    list = conf->widdbrel_list;
    while(list) {
      sql_wid_db_rel_build_xml_tree((WidRelData *)(list->data), toptree, 
				    conf);
      list = g_slist_next(list);
    }

    /* all the SqlQuery objects */
    list = conf->queries;
    while (list) {
      sql_query_build_xml_tree(SQL_QUERY(list->data), conf, doc);
      list = g_slist_next(list);
    }

    xmlSaveFile(conf->working_file, doc);
    xmlFreeDoc (doc);
    conf->save_up_to_date = TRUE;
  }
  else
    if (!conf->save_up_to_date)
      file_save_as_cb(NULL, data);
}


void file_saveas_dlg_ok_cb(GtkWidget *widget, void *data)
{
  gASQL_Main_Config *conf;
  gchar *txt, *filetxt;
  
  conf = (gASQL_Main_Config *) gtk_object_get_data(GTK_OBJECT(data), "conf");
  filetxt =g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(data)));
  gtk_object_destroy(GTK_OBJECT(data));

  if (g_file_test(filetxt, G_FILE_TEST_ISDIR)) {
    txt = g_strdup_printf(_("File '%s' is a directory!\n"), filetxt);
    gnome_error_dialog(txt);
    g_free(txt);
    return;
  }
  else {
    if (conf->working_file)
      g_free(conf->working_file);
    conf->working_file = filetxt;
    txt = gasql_get_title(conf);
    gtk_window_set_title(GTK_WINDOW(conf->app), txt);
    g_free(txt);    
    file_save_cb(NULL, conf);
  }
}


/* widget not used */
void file_save_as_cb (GtkWidget *widget, void *data)
{
  gASQL_Main_Config *conf;
  GtkWidget *dlg;

  conf = (gASQL_Main_Config *) data;
  /* let's now open a dialog... */
  dlg = gtk_file_selection_new(_("File to save to"));
  gtk_window_set_modal(GTK_WINDOW(dlg), TRUE);
  gtk_object_set_data(GTK_OBJECT(dlg), "conf", conf);
  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(dlg)->cancel_button), 
		     "clicked", 
		     GTK_SIGNAL_FUNC(file_open_dlg_cancel_cb), dlg); /* ! */
  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(dlg)->ok_button), 
		     "clicked", 
		     GTK_SIGNAL_FUNC(file_saveas_dlg_ok_cb), dlg);
  gtk_widget_show(dlg);
}

void file_close_cb (GtkWidget *widget, void *data)
{
  gASQL_Main_Config *conf;
  gchar *str;

  conf = (gASQL_Main_Config *) data;  
  if (conf->working_file) {
    g_free(conf->working_file);
    conf->working_file = NULL;
    str = gasql_get_title(conf);
    gtk_window_set_title(GTK_WINDOW(conf->app), str);
    g_free(str);
  }
}

void sql_conf_update_save_flag(GtkObject *widget, void *data)
{
  gASQL_Main_Config *conf;

  conf = (gASQL_Main_Config *) data;  
  conf->save_up_to_date = FALSE;
}


/* CB to display sth when updating the DB structure */
void progress_cb(GtkObject *obj, gchar *msg, guint now, guint total, 
		 gASQL_Main_Config *conf)
{
  if (msg)
    {
      gnome_appbar_set_status(GNOME_APPBAR(conf->appbar), msg);
      if (total==0)
	total = now+10; /* FIXME! */
      gnome_appbar_set_progress(GNOME_APPBAR(conf->appbar), 
				(gfloat)now / (gfloat)total);
    }
  else
    {
      gnome_appbar_clear_stack(GNOME_APPBAR(conf->appbar));
      gnome_appbar_set_progress(GNOME_APPBAR(conf->appbar), 0.);
    }
  gtk_main_iteration();
}

/***************************************************************************/
/*                                                                         */
/* Plugins callbacks                                                       */
/*                                                                         */
/***************************************************************************/
/*
 * CALLBACKS for the 1st page
 */ 

static void refresh_plugins_table_cb(GtkObject *obj, gASQL_Main_Config *conf);
void rescan_display_plugins_cb(GtkObject *obj, gASQL_Main_Config *conf)
{
  sql_access_rescan_display_plugins(conf->srv, conf->plugins_dir);
  if (conf->config_plugins_dlg)
    refresh_plugins_table_cb(NULL, conf);
}

struct foreach_hash_struct 
{
  gpointer find_value;
  gboolean found;
};

static void foreach_hash_cb(gpointer key, gpointer value, 
			    struct foreach_hash_struct *user_data)
{
  if (value == user_data->find_value)
    user_data->found = TRUE;
}


/* tells if a plugin is being used or not */
static gboolean is_plugin_used(gASQL_Main_Config *conf, SqlDataDisplayFns *fns)
{
  struct foreach_hash_struct *fhs;
  fhs = g_new(struct foreach_hash_struct ,1);
  fhs->find_value = fns;
  fhs->found = FALSE;

  g_hash_table_foreach(conf->srv->types_objects_hash, 
		       (GHFunc)(foreach_hash_cb), fhs);

  return fhs->found;
}

static void refresh_plugins_table_cb(GtkObject *obj, gASQL_Main_Config *conf)
{
  GSList *list;
  gchar *text[5];
  SqlDataDisplayFns *fns;
  GtkCList *clist;
  gint row;

  clist = gtk_object_get_data(GTK_OBJECT(conf->config_plugins_dlg), "clist");
  list = conf->srv->data_types_display;
  gtk_clist_freeze(clist);
  while (list)
    {
      fns = (SqlDataDisplayFns *)(list->data);
      if (fns->plugin_name) /* making sure it is a plugin */
	{
	  row = gtk_clist_find_row_from_data(clist, list->data);
	  text[0] = fns->plugin_name;
	  text[1] = fns->version;

	  if (is_plugin_used(conf, fns))
	    text[2] = _("Yes");
	  else
	    text[2] = _("No");
	  text[3] = fns->descr;
	  text[4] = fns->plugin_file;
	  if (row >= 0)
	    {
	      gtk_clist_remove(clist, row);
	      gtk_clist_insert(clist, row, text);
	    }
	  else
	    row = gtk_clist_append(clist, text);
	  gtk_clist_set_row_data(clist, row, fns);
	}
      list = g_slist_next(list);
    }

  /* removing any row not related to a plugin anymore */
  row = 0;
  while (row < clist->rows)
    {
      fns = gtk_clist_get_row_data(clist, row);
      if (!g_slist_find(conf->srv->data_types_display, fns))
	gtk_clist_remove(clist, row);
      else 
	row++;
    }

  gtk_clist_thaw(clist);
}

static void change_location_cancel_cb(GtkObject *obj, gASQL_Main_Config *conf)
{
  GtkWidget *fw;
  fw = gtk_object_get_data(GTK_OBJECT(conf->config_plugins_dlg), "fw");
  gtk_object_set_data(GTK_OBJECT(conf->config_plugins_dlg), "fw", NULL);
  gtk_widget_destroy(fw);
}

static void change_location_ok_cb(GtkObject *obj, gASQL_Main_Config *conf)
{
  gchar *str, *ptr;
  GtkWidget *fw;
  GtkWidget *wid;

  fw = gtk_object_get_data(GTK_OBJECT(conf->config_plugins_dlg), "fw");
  str = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fw)));
  /* removing trailing caracters in case we have a file instead of a dir */
  ptr = str+strlen(str)-1;
  while (*ptr != '/')
    {
      *ptr = '\0';
      ptr = str+strlen(str)-1;
    }
  *ptr = '\0';
  if (conf->plugins_dir)
    g_free(conf->plugins_dir);
  conf->plugins_dir = g_strdup(str);
  g_free(str);
  /* refreshing the display */
  wid = gtk_object_get_data(GTK_OBJECT(conf->config_plugins_dlg), "loclabel");
  gtk_label_set_text(GTK_LABEL(wid), conf->plugins_dir);
  rescan_display_plugins_cb(NULL, conf);

  gtk_object_set_data(GTK_OBJECT(conf->config_plugins_dlg), "fw", NULL);
  gtk_widget_destroy(fw);
}

static void change_location_cb(GtkObject *obj, gASQL_Main_Config *conf)
{
  GtkWidget *fw;

  fw = gtk_object_get_data(GTK_OBJECT(conf->config_plugins_dlg), "fw");
  if (!fw)
    {
      fw = gtk_file_selection_new("Select plugins directory");
      gtk_object_set_data(GTK_OBJECT(conf->config_plugins_dlg), "fw", fw);
      gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(fw));
      gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(fw)->cancel_button), 
			 "clicked", 
			 GTK_SIGNAL_FUNC(change_location_cancel_cb), conf);
      gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(fw)->ok_button), 
			 "clicked", 
			 GTK_SIGNAL_FUNC(change_location_ok_cb), conf);
      gtk_widget_show(fw);
    }
  else
    gdk_window_raise(fw->window);
}

static void notify_destroy_config_plugins_dlg_cb(GtkWidget *wid, 
						 gASQL_Main_Config *conf)
{
  conf->config_plugins_dlg = NULL;
}

/*
 * CALLBACKS for the 2nd page
 */

/* refreshes the clist with the data types */
static void data_types_updated_cb(GtkObject *obj, gASQL_Main_Config *conf)
{
  gint row;
  GtkCList *clist;
  gpointer data;
  GSList *list;

  clist = gtk_object_get_data(GTK_OBJECT(conf->config_plugins_dlg), "tclist");

  /* add the ones not yet there */
  list = conf->srv->data_types;
  while (list)
    {
      row = gtk_clist_find_row_from_data(clist, list->data);
      if (row < 0) /* need to be added */
	{
	  row = gtk_clist_append(clist,
				 &(SQL_DATA_TYPE(list->data)->sqlname));
	  gtk_clist_set_row_data(clist, row, list->data);
	}
      list = g_slist_next(list);
    }

  /* remove the ones not present anymore */
  row = 0;
  while (row < clist->rows)
    {
      data = gtk_clist_get_row_data(clist, row);
      if (!g_slist_find(conf->srv->data_types, data))
	gtk_clist_remove(clist, row);
      else
	row++;
    }
  refresh_plugins_table_cb(NULL, conf);
}

static void plugins_ch_combo_sel_changed_cb(GtkObject *obj, gpointer *newsel,
					    gASQL_Main_Config *conf)
{
  gpointer ptr;
  SqlDataType *dt;

  dt = gtk_object_get_data(GTK_OBJECT(conf->config_plugins_dlg), "dt");
  ptr = g_hash_table_lookup(conf->srv->types_objects_hash, dt);
  if (ptr)
    g_hash_table_remove(conf->srv->types_objects_hash, dt);
  if (newsel)
    g_hash_table_insert(conf->srv->types_objects_hash, dt, newsel);
  refresh_plugins_table_cb(NULL, conf);
}


/* select cb from the data types clist */
static void data_types_clist_select_cb(GtkWidget *widget, gint row, gint col,
				       GdkEventButton *event, 
				       gASQL_Main_Config *conf)
{
  GtkWidget *wid, *clist;
  SqlDataType *dt;
  GSList *blist, *list;
  SqlDataDisplayFns *fns;
  gint i;

  clist = gtk_object_get_data(GTK_OBJECT(conf->config_plugins_dlg), "tclist");
  dt = gtk_clist_get_row_data(GTK_CLIST(clist), row);
  if (dt)
    {
      gtk_object_set_data(GTK_OBJECT(conf->config_plugins_dlg), "dt", dt);
      wid = gtk_object_get_data(GTK_OBJECT(conf->config_plugins_dlg), 
				"seltname");
      gtk_label_set_text(GTK_LABEL(wid), dt->sqlname);
      wid = gtk_object_get_data(GTK_OBJECT(conf->config_plugins_dlg), 
				"seltdescr");
      gtk_label_set_text(GTK_LABEL(wid), dt->descr);
      /* computing the list for choice combo from tge gda types accepted
       by each plugin */
      wid = gtk_object_get_data(GTK_OBJECT(conf->config_plugins_dlg), "selcc");
      blist = g_slist_append(NULL, NULL);
      list = conf->srv->data_types_display;
      while (list)
	{
	  fns = (SqlDataDisplayFns *)(list->data);
	  if (fns->plugin_name)
	    {
  	      gboolean found;
	      found = FALSE;
	      i = 0;
	      if (fns->nb_gda_type == 0) /* any Gda Type is OK */
		found = TRUE;
	      while ((i<fns->nb_gda_type) && !found)
		{
		  if (fns->valid_gda_types[i] == dt->gda_type)
		    found = TRUE;
		  i++;
		}
	      if (found)
		blist = g_slist_append(blist, fns);
	    }
	  list = g_slist_next(list);
	}
      choice_combo_set_content(CHOICE_COMBO(wid), blist, 
			       GTK_STRUCT_OFFSET(SqlDataDisplayFns, 
						 plugin_name));
      /* if a plugin is already bound, set it as default */
      fns = g_hash_table_lookup(conf->srv->types_objects_hash, dt);
      if (fns)
	{
	  i = g_slist_index(blist, fns);
	  if (i>=0)
	    choice_combo_set_selection_num(CHOICE_COMBO(wid), i);
	}
      g_slist_free(blist);
      gtk_signal_connect(GTK_OBJECT(wid), "selection_changed",
			 GTK_SIGNAL_FUNC(plugins_ch_combo_sel_changed_cb), 
			 conf);
    }
  else
    gtk_object_set_data(GTK_OBJECT(conf->config_plugins_dlg), "dt", NULL);
}

static void data_types_clist_unselect_cb(GtkWidget *widget, gint row, gint col,
					 GdkEventButton *event, 
					 gASQL_Main_Config *conf)
{
  GtkWidget *wid;

  wid = gtk_object_get_data(GTK_OBJECT(conf->config_plugins_dlg), "seltname");
  gtk_label_set_text(GTK_LABEL(wid), _("<Select one>"));
  wid = gtk_object_get_data(GTK_OBJECT(conf->config_plugins_dlg), "seltdescr");
  gtk_label_set_text(GTK_LABEL(wid), _("<NONE>"));

  /* choice combo */
  wid = gtk_object_get_data(GTK_OBJECT(conf->config_plugins_dlg), "selcc");
  gtk_signal_disconnect_by_func(GTK_OBJECT(wid),
			 GTK_SIGNAL_FUNC(plugins_ch_combo_sel_changed_cb), 
				conf);
  choice_combo_set_content(CHOICE_COMBO(wid), NULL, 0);
  gtk_object_set_data(GTK_OBJECT(conf->config_plugins_dlg), "dt", NULL);
}


/*
 * PROPS dialog buinding
 */
void config_display_plugins_cb(GtkObject *obj, gASQL_Main_Config *conf)
{
  GtkWidget *props, *label, *page, *wid, *sw, *box, *table, *box2, *clist;
  GSList *list;

  if (conf->config_plugins_dlg) {
    gdk_window_raise(conf->config_plugins_dlg->window);
    return;
  }

  props = gnome_property_box_new();
  conf->config_plugins_dlg = props;

  /* 
   * known plugins and their usage: 1st page
   */
  label = gtk_label_new(_("Plugins list"));
  gtk_widget_show(label);
  page = gtk_hbox_new(FALSE, GNOME_PAD);
  gtk_object_set_data(GTK_OBJECT(props), "pluglist", page);

  /* on the left is a clist in its sw and the location of plugins */
  box = gtk_vbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(page), box, TRUE, TRUE, GNOME_PAD);
  gtk_widget_show(box);

  box2 = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(box), box2, FALSE, TRUE, GNOME_PAD);
  gtk_widget_show(box2);
  wid = gtk_label_new(_("Plugins location:"));
  gtk_box_pack_start(GTK_BOX(box2), wid, FALSE, TRUE, GNOME_PAD);
  gtk_widget_show(wid);
  wid = gtk_label_new(conf->plugins_dir);
  gtk_object_set_data(GTK_OBJECT(props), "loclabel", wid);
  gtk_box_pack_start(GTK_BOX(box2), wid, FALSE, TRUE, GNOME_PAD);
  gtk_widget_show(wid);

  sw = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
				 GTK_POLICY_AUTOMATIC,
				 GTK_POLICY_AUTOMATIC);
  gtk_box_pack_start(GTK_BOX(box), sw, TRUE, TRUE, GNOME_PAD);
  gtk_widget_show(sw);
  clist = gtk_clist_new(5);
  gtk_object_set_data(GTK_OBJECT(conf->config_plugins_dlg), "clist", clist);
  gtk_clist_set_column_title(GTK_CLIST(clist), 0, _("Plugin"));
  gtk_clist_set_column_title(GTK_CLIST(clist), 1, _("Version"));
  gtk_clist_set_column_title(GTK_CLIST(clist), 2, _("Used"));
  gtk_clist_set_column_title(GTK_CLIST(clist), 3, _("Description"));
  gtk_clist_set_column_title(GTK_CLIST(clist), 4, _("File"));
  gtk_clist_set_column_auto_resize(GTK_CLIST(clist), 0, TRUE);
  gtk_clist_set_column_auto_resize(GTK_CLIST(clist), 1, TRUE);
  gtk_clist_set_column_auto_resize(GTK_CLIST(clist), 2, TRUE);
  gtk_clist_set_column_auto_resize(GTK_CLIST(clist), 3, TRUE);
  gtk_clist_set_column_auto_resize(GTK_CLIST(clist), 4, TRUE);
  gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_SINGLE);
  gtk_clist_column_titles_show(GTK_CLIST(clist));
  gtk_clist_column_titles_passive(GTK_CLIST(clist));
  gtk_container_add(GTK_CONTAINER(sw), clist);
  gtk_widget_show(clist);

  /* on the right are some buttons and the directory location of the plugins */
  box = gtk_vbutton_box_new();
  gtk_button_box_set_layout(GTK_BUTTON_BOX(box), GTK_BUTTONBOX_SPREAD);
  gtk_box_pack_start(GTK_BOX(page), box, FALSE, FALSE, GNOME_PAD);
  gtk_widget_show(box);
  wid = gtk_button_new_with_label(_("Change location"));
  gtk_signal_connect(GTK_OBJECT(wid), "clicked",
		     GTK_SIGNAL_FUNC(change_location_cb),
		     conf);
  gtk_container_add(GTK_CONTAINER(box), wid);
  gtk_widget_show(wid);
  wid = gtk_button_new_with_label(_("Rescan plugins"));
  gtk_signal_connect(GTK_OBJECT(wid), "clicked",
		     GTK_SIGNAL_FUNC(rescan_display_plugins_cb), 
		     conf);
  gtk_container_add(GTK_CONTAINER(box), wid);
  gtk_widget_show(wid);
  wid = gtk_button_new_with_label(_("Refresh"));
  gtk_container_add(GTK_CONTAINER(box), wid);
  gtk_widget_show(wid);
  gtk_signal_connect(GTK_OBJECT(wid), "clicked",
		     GTK_SIGNAL_FUNC(refresh_plugins_table_cb),
		     conf);

  gnome_property_box_append_page(GNOME_PROPERTY_BOX(props), page, label);
  gtk_container_set_border_width(GTK_CONTAINER(page), GNOME_PAD);
  refresh_plugins_table_cb(NULL, conf);
  gtk_widget_show(page);

  /* 
   * Data types bindings: 2nd page
   */
  label = gtk_label_new(_("Data Type bindings"));
  gtk_widget_show(label);
  page = gtk_hbox_new(FALSE, GNOME_PAD);
  gtk_object_set_data(GTK_OBJECT(props), "dtbinding", page);

  /* on the left side is a clist in a sw */
  sw = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
				 GTK_POLICY_AUTOMATIC,
				 GTK_POLICY_AUTOMATIC);
  gtk_box_pack_start(GTK_BOX(page), sw, TRUE, TRUE, GNOME_PAD);
  gtk_widget_show(sw);
  wid = gtk_clist_new(1);
  gtk_clist_set_column_title(GTK_CLIST(wid), 0, _("Data types"));
  gtk_clist_set_selection_mode(GTK_CLIST(wid), GTK_SELECTION_SINGLE);
  gtk_clist_column_titles_show(GTK_CLIST(wid));
  gtk_clist_column_titles_passive(GTK_CLIST(wid));
  gtk_container_add(GTK_CONTAINER(sw), wid);
  gtk_widget_show(wid);
  list = conf->srv->data_types;
  while (list)
    {
      gint row;
      row = gtk_clist_append(GTK_CLIST(wid),
			     &(SQL_DATA_TYPE(list->data)->sqlname));
      gtk_clist_set_row_data(GTK_CLIST(wid), row, list->data);
      list = g_slist_next(list);
    }
  gtk_object_set_data(GTK_OBJECT(conf->config_plugins_dlg), "tclist", wid);
  gtk_signal_connect(GTK_OBJECT(wid), "select_row",
		     GTK_SIGNAL_FUNC(data_types_clist_select_cb), conf);
  gtk_signal_connect(GTK_OBJECT(wid), "unselect_row",
		     GTK_SIGNAL_FUNC(data_types_clist_unselect_cb), conf);
  gtk_signal_connect_while_alive(GTK_OBJECT(conf->srv), "data_types_updated",
				 GTK_SIGNAL_FUNC(data_types_updated_cb), conf,
				 GTK_OBJECT(wid));

  /* on the right side is the action area */
  box = gtk_vbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(page), box, TRUE, TRUE, GNOME_PAD);

  table = gtk_table_new(3, 2, FALSE);
  gtk_table_set_col_spacing(GTK_TABLE(table), 0, GNOME_PAD);
  gtk_box_pack_start(GTK_BOX(box), table, FALSE, FALSE, 0);
  wid = gtk_label_new(_("Data Type:"));
  gtk_table_attach_defaults(GTK_TABLE(table), wid, 0, 1, 0, 1);
  gtk_widget_show(wid);
  wid = gtk_label_new(_("<Select one>"));
  gtk_object_set_data(GTK_OBJECT(conf->config_plugins_dlg), "seltname",
		      wid);
  gtk_table_attach_defaults(GTK_TABLE(table), wid, 1, 2, 0, 1);
  gtk_widget_show(wid);
  wid = gtk_label_new(_("Description:"));
  gtk_table_attach_defaults(GTK_TABLE(table), wid, 0, 1, 1, 2);
  gtk_widget_show(wid);
  wid = gtk_label_new(_("<NONE>"));
  gtk_object_set_data(GTK_OBJECT(conf->config_plugins_dlg), "seltdescr",
		      wid);
  gtk_table_attach_defaults(GTK_TABLE(table), wid, 1, 2, 1, 2);
  gtk_widget_show(wid);
  wid = gtk_label_new(_("Display Plugin:"));
  gtk_table_attach_defaults(GTK_TABLE(table), wid, 0, 1, 2, 3);
  gtk_widget_show(wid);
  wid = choice_combo_new();
  gtk_object_set_data(GTK_OBJECT(conf->config_plugins_dlg), "selcc",
		      wid);
  choice_combo_set_content(CHOICE_COMBO(wid), NULL, 0);
  gtk_table_attach_defaults(GTK_TABLE(table), wid, 1, 2, 2, 3);
  gtk_widget_show(wid);
  gtk_widget_show(table);
  gtk_widget_show(box);

  gnome_property_box_append_page(GNOME_PROPERTY_BOX(props), page, label);
  gtk_container_set_border_width(GTK_CONTAINER(page), GNOME_PAD);
  gtk_widget_show(page);

  /* 
   * Fine tuned plugins usage
   */
  label = gtk_label_new(_("Individual objects bindings"));
  gtk_widget_show(label);
  page = gtk_hbox_new(FALSE, GNOME_PAD);
  gtk_object_set_data(GTK_OBJECT(props), "objbinding", page);

  wid = gtk_label_new("This page will allow you to bind a plugin to\n"
		      "individual table's fields for more precision\n"
		      "of how to edit and display your data.");
  gtk_box_pack_start(GTK_BOX(page), wid, TRUE, TRUE, GNOME_PAD);
  gtk_widget_show(wid);

  gnome_property_box_append_page(GNOME_PROPERTY_BOX(props), page, label);
  gtk_container_set_border_width(GTK_CONTAINER(page), GNOME_PAD);
  gtk_widget_show(page);


  gtk_window_set_title(GTK_WINDOW(props), _("Plugins preferences"));
  gtk_widget_set_usize(props, 600, 300);
  gtk_widget_hide(GNOME_PROPERTY_BOX(props)->apply_button);
  gtk_signal_connect(GTK_OBJECT(props), "destroy",
		     GTK_SIGNAL_FUNC(notify_destroy_config_plugins_dlg_cb), 
		     conf);
  gtk_widget_show(props);  
}

/* Configuration of the DBMS using bonobo controls */
#ifndef HAVE_BONOBO
void bonobo_needed_cb (GtkWidget *widget, gASQL_Main_Config *conf)
{
  gnome_error_dialog_parented(_("Bonobo support is needed for "
				"this operation!"),
			      GTK_WINDOW(conf->app));     
}
#endif

#ifdef HAVE_BONOBO
/* returns a GnomeDbControlWidget if successfull and NULL 
 * (and displays an error message) otherwise
 */
static GtkWidget *get_gnome_db_control_widget(gchar *goad_id,
					      gASQL_Main_Config *conf)
{
  GtkWidget *control_widget=NULL;

  if (gnome_db_control_widget_supported(goad_id))
    {
      control_widget = gnome_db_control_widget_new(goad_id);
      if (!GNOME_DB_CONTROL_WIDGET(control_widget)->bonobo_widget)
	{
	  gtk_object_destroy(GTK_OBJECT(control_widget));
	  gnome_error_dialog_parented(_("The Bonobo control "
					"for this operation "
					"cannot be loaded"),
				      GTK_WINDOW(conf->app));     
	  control_widget = NULL;
	}
    }
  else
    gnome_error_dialog_parented(_("The Bonobo control for this operation "
				  "cannot be found"),
				GTK_WINDOW(conf->app));     

  return control_widget;
}

static void bonobo_dialog_clicked_cb(GtkWidget *dlg, gint button_number,
				     gASQL_Main_Config *conf)
{
  gtk_object_destroy(GTK_OBJECT(dlg));
  if (conf->users_list_dlg == dlg)
    conf->users_list_dlg = NULL;
  if (conf->users_acl_dlg == dlg)
    conf->users_acl_dlg = NULL;
}
#endif

void users_settings_cb(GtkObject *obj, gASQL_Main_Config *conf)
{
#ifndef HAVE_BONOBO
  bonobo_needed_cb(obj, conf);
#else
  if (conf->users_list_dlg)
    gdk_window_raise(conf->users_list_dlg->window);    
  else
    {
      if (conf->srv->gda_datasource &&
	  (*(conf->srv->gda_datasource->str) != '\0'))
	{
	  gchar *bonobo_name;/* = "control:gnome-db-users-ac";*/
	  GtkWidget *control_widget;
	  GtkWidget *dlg;
	  gchar *str;
	  gchar *dsn;

	  str = g_strdup_printf("/gdalib/%s/", 
				conf->srv->gda_datasource->str);
	  gnome_config_push_prefix(str);
	  g_free(str);
	  str = gnome_config_get_string("Provider");
	  dsn = gnome_config_get_string("DSN");
	  gnome_config_pop_prefix();
	  bonobo_name = g_strdup_printf("control:%s-users-list", str);
	  g_free(str);
	  control_widget = get_gnome_db_control_widget(bonobo_name, conf);
	  if (control_widget)
	    {
	      dlg = gnome_dialog_new(_("Users settings"), 
				     GNOME_STOCK_BUTTON_OK, NULL);
	      gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dlg)->vbox), 
				 control_widget, 
				 TRUE, TRUE, 0);
	      gtk_widget_set_usize(dlg, 450, 450);
	      gtk_signal_connect(GTK_OBJECT(dlg), "clicked",
				 GTK_SIGNAL_FUNC(bonobo_dialog_clicked_cb), 
				 conf);
	      conf->users_list_dlg = dlg;
	      
	      /* setting some basic properties */
	      gnome_db_control_widget_set_prop
		(GNOME_DB_CONTROL_WIDGET(control_widget), 
		 GNOME_DB_CONTROL_PROP_DSN, dsn);
	      gnome_db_control_widget_set_prop
		(GNOME_DB_CONTROL_WIDGET(control_widget), 
		 GNOME_DB_CONTROL_PROP_IDENT_NAME, 
		 conf->srv->user_name->str);
	      gnome_db_control_widget_set_prop
		(GNOME_DB_CONTROL_WIDGET(control_widget), 
		 GNOME_DB_CONTROL_PROP_IDENT_PASS, 
		 conf->srv->password->str);
	      gnome_db_control_widget_set_prop
		(GNOME_DB_CONTROL_WIDGET(control_widget), 
		 GNOME_DB_CONTROL_PROP_IDENT_SHOW, FALSE); 
	      gnome_db_control_widget_set_prop
		(GNOME_DB_CONTROL_WIDGET(control_widget), 
		 GNOME_DB_CONTROL_PROP_START, TRUE); 
	      /* starting the widget to display usefull data */
	      
	      gtk_widget_show_all(dlg);
	    }
	  if (dsn)
	    g_free(dsn);
	  g_free(bonobo_name);
	}
      else
	{
	  options_config_cb (NULL, conf);
	  /* sets the "Apply" button active because we consider 
	     a modif done */
	  gnome_property_box_changed(GNOME_PROPERTY_BOX(conf->config_dlg));
	}
    }
#endif
}


void users_access_cb(GtkObject *obj, gASQL_Main_Config *conf)
{
#ifndef HAVE_BONOBO
  bonobo_needed_cb(obj, conf);
#else
  if (conf->users_acl_dlg)
    gdk_window_raise(conf->users_acl_dlg->window);    
  else
    {
      if (conf->srv->gda_datasource &&
	  (*(conf->srv->gda_datasource->str) != '\0'))
	{
	  gchar *bonobo_name;/* = "control:gnome-db-users-ac";*/
	  GtkWidget *control_widget;
	  GtkWidget *dlg;
	  gchar *str;
	  gchar *dsn;

	  str = g_strdup_printf("/gdalib/%s/", 
				conf->srv->gda_datasource->str);
	  gnome_config_push_prefix(str);
	  g_free(str);
	  str = gnome_config_get_string("Provider");
	  dsn = gnome_config_get_string("DSN");
	  gnome_config_pop_prefix();
	  bonobo_name = g_strdup_printf("control:%s-users-ac", str);
	  g_free(str);
	  control_widget = get_gnome_db_control_widget(bonobo_name, conf);
	  if (control_widget)
	    {
	      dlg = gnome_dialog_new(_("Users access rights"), 
				     GNOME_STOCK_BUTTON_OK, NULL);
	      gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dlg)->vbox), 
				 control_widget, 
				 TRUE, TRUE, 0);
	      gtk_widget_set_usize(dlg, 500, 450);
	      gtk_signal_connect(GTK_OBJECT(dlg), "clicked",
				 GTK_SIGNAL_FUNC(bonobo_dialog_clicked_cb), 
				 conf);
	      conf->users_acl_dlg = dlg;
	      
	      /* setting some basic properties */
	      gnome_db_control_widget_set_prop
		(GNOME_DB_CONTROL_WIDGET(control_widget), 
		 GNOME_DB_CONTROL_PROP_DSN, dsn);
	      gnome_db_control_widget_set_prop
		(GNOME_DB_CONTROL_WIDGET(control_widget), 
		 GNOME_DB_CONTROL_PROP_IDENT_NAME, 
		 conf->srv->user_name->str);
	      gnome_db_control_widget_set_prop
		(GNOME_DB_CONTROL_WIDGET(control_widget), 
		 GNOME_DB_CONTROL_PROP_IDENT_PASS, 
		 conf->srv->password->str);
	      gnome_db_control_widget_set_prop
		(GNOME_DB_CONTROL_WIDGET(control_widget), 
		 GNOME_DB_CONTROL_PROP_IDENT_SHOW, FALSE); 
	      gnome_db_control_widget_set_prop
		(GNOME_DB_CONTROL_WIDGET(control_widget), 
		 GNOME_DB_CONTROL_PROP_START, TRUE); 
	      /* starting the widget to display usefull data */
	      
	      gtk_widget_show_all(dlg);
	    }
	  if (dsn)
	    g_free(dsn);
	  g_free(bonobo_name);
	}
      else
	{
	  options_config_cb (NULL, conf);
	  /* sets the "Apply" button active because we consider 
	     a modif done */
	  gnome_property_box_changed(GNOME_PROPERTY_BOX(conf->config_dlg));
	}
    }
#endif
}

void users_groups_cb(GtkObject *obj, gASQL_Main_Config *conf)
{
#ifndef HAVE_BONOBO
  bonobo_needed_cb(obj, conf);
#else
  if (conf->users_groups_dlg)
    gdk_window_raise(conf->users_groups_dlg->window);    
  else
    {
      if (conf->srv->gda_datasource &&
	  (*(conf->srv->gda_datasource->str) != '\0'))
	{
	  gchar *bonobo_name;/* = "control:gnome-db-users-groups";*/
	  GtkWidget *control_widget;
	  GtkWidget *dlg;
	  gchar *str;
	  gchar *dsn;

	  str = g_strdup_printf("/gdalib/%s/", 
				conf->srv->gda_datasource->str);
	  gnome_config_push_prefix(str);
	  g_free(str);
	  str = gnome_config_get_string("Provider");
	  dsn = gnome_config_get_string("DSN");
	  gnome_config_pop_prefix();
	  bonobo_name = g_strdup_printf("control:%s-users-groups", str);
	  g_free(str);
	  control_widget = get_gnome_db_control_widget(bonobo_name, conf);
	  if (control_widget)
	    {
	      dlg = gnome_dialog_new(_("Users groups"), 
				     GNOME_STOCK_BUTTON_OK, NULL);
	      gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dlg)->vbox), 
				 control_widget, 
				 TRUE, TRUE, 0);
	      gtk_widget_set_usize(dlg, 500, 450);
	      gtk_signal_connect(GTK_OBJECT(dlg), "clicked",
				 GTK_SIGNAL_FUNC(bonobo_dialog_clicked_cb), 
				 conf);
	      conf->users_groups_dlg = dlg;
	      
	      /* setting some basic properties */
	      gnome_db_control_widget_set_prop
		(GNOME_DB_CONTROL_WIDGET(control_widget), 
		 GNOME_DB_CONTROL_PROP_DSN, dsn);
	      gnome_db_control_widget_set_prop
		(GNOME_DB_CONTROL_WIDGET(control_widget), 
		 GNOME_DB_CONTROL_PROP_IDENT_NAME, 
		 conf->srv->user_name->str);
	      gnome_db_control_widget_set_prop
		(GNOME_DB_CONTROL_WIDGET(control_widget), 
		 GNOME_DB_CONTROL_PROP_IDENT_PASS, 
		 conf->srv->password->str);
	      gnome_db_control_widget_set_prop
		(GNOME_DB_CONTROL_WIDGET(control_widget), 
		 GNOME_DB_CONTROL_PROP_IDENT_SHOW, FALSE); 
	      gnome_db_control_widget_set_prop
		(GNOME_DB_CONTROL_WIDGET(control_widget), 
		 GNOME_DB_CONTROL_PROP_START, TRUE); 
	      /* starting the widget to display usefull data */
	      
	      gtk_widget_show_all(dlg);
	    }
	  if (dsn)
	    g_free(dsn);
	  g_free(bonobo_name);
	}
      else
	{
	  options_config_cb (NULL, conf);
	  /* sets the "Apply" button active because we consider 
	     a modif done */
	  gnome_property_box_changed(GNOME_PROPERTY_BOX(conf->config_dlg));
	}
    }
#endif
}
