/* mainpagequery.c
 *
 * Copyright (C) 1999 Vivien Malerba
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include <config.h>
#include "mainpagequery.h"
#include "sqlquery.h"
#include "sqlquerywid.h"
#include "sqlqueryexec.h"

static void      main_page_query_class_init      (MainPageQueryClass *class);
static void      main_page_query_init            (MainPageQuery *wid);
static void      main_page_query_initialize      (MainPageQuery *wid);

static void      selection_made(GtkWidget *wid,
				gint row,
				gint column,
				GdkEventButton *event,
				gpointer data);

static void clist_remove_a_query(MainPageQuery *mpq, gint row);

/* buttons callbacks */
static void create_query_button_cb(GtkWidget *button, gpointer data);
static void drop_query_button_cb(GtkWidget *button, gpointer data);
static void edit_query_button_cb(GtkWidget *button, gpointer data);
static void exec_query_button_cb(GtkWidget *button, gpointer data);

static void edit_query_query_changed_cb(SqlQuery *q, gpointer data);

/* connection stopping callback */
static void conn_closed_cb(GtkWidget *serveur, MainPageQuery *mpq);

typedef struct {
  SqlQuery *query;
  GtkWidget *edit_dlg;
  GtkWidget *exec_dlg;
} Row_Data;

guint main_page_query_get_type(void) 
{
  static guint f_type = 0;
  
  if (!f_type)
    {
      GtkTypeInfo f_info =
      {
        "Main_Page_Query",
        sizeof (MainPageQuery),	
        sizeof (MainPageQueryClass),
        (GtkClassInitFunc) main_page_query_class_init,
        (GtkObjectInitFunc) main_page_query_init,
        (GtkArgSetFunc) NULL,
        (GtkArgGetFunc) NULL
      };
      
      f_type = gtk_type_unique (gtk_vbox_get_type (), &f_info);
    }
  
  return f_type;   
}

static void main_page_query_class_init (MainPageQueryClass *class)
{
  GtkObjectClass *object_class;
  
  object_class = (GtkObjectClass*) class;
}

/* Callback to handle the button presses over the CList */
static gint press_handler_event(GtkWidget *widget,
				GdkEventButton *event, MainPageQuery *mpq)
{
  gint row, col;

  if (GTK_IS_CLIST(widget)) {
    /* setting the right selection */
    gtk_clist_get_selection_info(GTK_CLIST(widget),
				 event->x, event->y,
				 &row, &col);
    if ((mpq->sel_row != row) && (event->button != 1)) 
      gtk_clist_select_row(GTK_CLIST(widget), row, col);    

    /* depending on what the clicks were */
    if (event->type==GDK_2BUTTON_PRESS) {
      Row_Data *rd;

      rd = (Row_Data *) gtk_clist_get_row_data(GTK_CLIST(widget), row);
      exec_query_button_cb(NULL, mpq);
      return TRUE;
    }

    if (event->button == 3) {
      GtkWidget *menu, *wid;

      menu = gtk_menu_new();
      wid = gtk_menu_item_new_with_label(_("Properties"));
      gtk_signal_connect(GTK_OBJECT(wid), "activate",
			 GTK_SIGNAL_FUNC(edit_query_button_cb), mpq);
      gtk_menu_append(GTK_MENU(menu), wid);
      gtk_widget_show(wid);
      wid = gtk_menu_item_new_with_label(_("Execute"));
      gtk_signal_connect(GTK_OBJECT(wid), "activate",
			 GTK_SIGNAL_FUNC(exec_query_button_cb), mpq);
      gtk_menu_append(GTK_MENU(menu), wid);
      gtk_widget_show(wid);
      gtk_menu_popup (GTK_MENU(menu), NULL, NULL, NULL, NULL,
		      event->button, event->time);
      /* Tell calling code that we have handled this event */
      return TRUE;
    }
  }

  return FALSE;
}


static void main_page_query_init(MainPageQuery *wid)
{
  GtkWidget *sw, *bb;

  /* Scrolled Window fro CList */
  sw = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
				 GTK_POLICY_AUTOMATIC,
				 GTK_POLICY_AUTOMATIC);
  gtk_box_pack_start(GTK_BOX(wid), sw, TRUE, TRUE, GNOME_PAD);
  gtk_widget_show(sw);

  /* CList */
  wid->clist = gtk_clist_new(2);
  gtk_clist_set_column_title(GTK_CLIST(wid->clist), 0, _("Query"));
  gtk_clist_set_column_title(GTK_CLIST(wid->clist), 1, _("Description"));
  gtk_clist_set_selection_mode(GTK_CLIST(wid->clist), GTK_SELECTION_SINGLE);
  gtk_clist_column_titles_show(GTK_CLIST(wid->clist));
  gtk_clist_column_titles_passive(GTK_CLIST(wid->clist));
  gtk_clist_set_column_auto_resize(GTK_CLIST(wid->clist), 0, TRUE);
  gtk_clist_set_column_auto_resize(GTK_CLIST(wid->clist), 1, TRUE);
  gtk_container_add(GTK_CONTAINER(sw), wid->clist);
  gtk_widget_show(wid->clist);
  gtk_signal_connect(GTK_OBJECT(wid->clist), "select_row",
		     GTK_SIGNAL_FUNC(selection_made), wid);
  gtk_signal_connect(GTK_OBJECT(wid->clist),
		     "button_press_event",
		     GTK_SIGNAL_FUNC(press_handler_event),
		     wid);
  
  /* Button Box */
  bb = gtk_hbutton_box_new();
  gtk_button_box_set_layout(GTK_BUTTON_BOX(bb), GTK_BUTTONBOX_SPREAD);
  gtk_box_pack_end(GTK_BOX(wid), bb, FALSE, TRUE, GNOME_PAD);
  gtk_widget_show(bb);

  /* Remove Query Button */
  wid->remove_query = gtk_button_new_with_label(_("Drop Query"));
  gtk_container_add(GTK_CONTAINER(bb), wid->remove_query);
  gtk_widget_show(wid->remove_query);
  gtk_signal_connect(GTK_OBJECT(wid->remove_query), "clicked",
		     GTK_SIGNAL_FUNC(drop_query_button_cb), wid);

  /* Edit Query Button */
  wid->edit_query = gtk_button_new_with_label(_("Properties"));
  gtk_container_add(GTK_CONTAINER(bb), wid->edit_query);
  gtk_widget_show(wid->edit_query);
  gtk_signal_connect(GTK_OBJECT(wid->edit_query), "clicked",
		     GTK_SIGNAL_FUNC(edit_query_button_cb), wid);

  /* View Query Button */
  wid->exec_query = gtk_button_new_with_label(_("Execute"));
  gtk_container_add(GTK_CONTAINER(bb), wid->exec_query);
  gtk_signal_connect(GTK_OBJECT(wid->exec_query), "clicked",
		     GTK_SIGNAL_FUNC(exec_query_button_cb), wid);
  gtk_widget_show(wid->exec_query);

  /* New Query Button */
  wid->new_query = gtk_button_new_with_label(_("Create Query"));
  gtk_container_add(GTK_CONTAINER(bb), wid->new_query);
  gtk_widget_show(wid->new_query);
  gtk_signal_connect(GTK_OBJECT(wid->new_query), "clicked",
		     GTK_SIGNAL_FUNC(create_query_button_cb), wid);

  wid->sel_row = -1;
  wid->new_dlg = NULL;
}

GtkWidget* main_page_query_new(gASQL_Main_Config *conf)
{
  GtkObject *obj;
  MainPageQuery *wid;

  obj =  gtk_type_new(main_page_query_get_type());
  wid = MAIN_PAGE_QUERY(obj);
  wid->conf = conf;

  main_page_query_initialize(wid);

  gtk_signal_connect(GTK_OBJECT(conf->srv), "conn_closed",
		     GTK_SIGNAL_FUNC(conn_closed_cb), wid);
  return GTK_WIDGET(obj);
}

static void main_page_query_initialize(MainPageQuery *wid)
{

  gtk_widget_set_sensitive(wid->remove_query, FALSE);
  gtk_widget_set_sensitive(wid->exec_query, FALSE);
  gtk_widget_set_sensitive(wid->edit_query, FALSE);
  gtk_widget_set_sensitive(wid->new_query, FALSE);
  gasql_conf_register_sensitive_on_connect(wid->conf, 
					   GTK_WIDGET(wid->new_query));
}

static void selection_made(GtkWidget *wid,
			   gint row,
			   gint column,
			   GdkEventButton *event,
			   gpointer data)
{
  MainPageQuery *mpq;

  mpq = MAIN_PAGE_QUERY(data);
  mpq->sel_row = row;
  gtk_widget_set_sensitive(mpq->remove_query, TRUE);
  gtk_widget_set_sensitive(mpq->edit_query, TRUE);
  gtk_widget_set_sensitive(mpq->exec_query, TRUE);
}

static void conn_closed_cb(GtkWidget *serveur, MainPageQuery *mpq)
{
  gint nb, i;

  nb = GTK_CLIST(mpq->clist)->rows;
  for (i=0; i<nb; i++)
    clist_remove_a_query(mpq, 0);
  /* close any add dialog */
  if (mpq->new_dlg) {
    gnome_dialog_close(GNOME_DIALOG(mpq->new_dlg));
    mpq->new_dlg = NULL;
  }
}

static void clist_remove_a_query(MainPageQuery *mpq, gint row)
{
  gchar *str;
  Row_Data *rd;
  GSList *list;
  WidRelData *reldata;

  if ((row >= GTK_CLIST(mpq->clist)->rows) || (row<0))
    return;

  /* remove entry from CList */
  rd = (Row_Data *) gtk_clist_get_row_data(GTK_CLIST(mpq->clist), row);
  gtk_clist_remove(GTK_CLIST(mpq->clist), row);

  if (rd) {
    /* if there is a SqlQueryWid and a dialog, it destroys itself before */
    if (rd->exec_dlg) {
      gnome_dialog_close(GNOME_DIALOG(rd->exec_dlg));
      rd->exec_dlg = NULL;
    }
    if (rd->edit_dlg) {
      gnome_dialog_close(GNOME_DIALOG(rd->edit_dlg));
      rd->edit_dlg = NULL;
    }
    if (rd->query) {
      mpq->conf->queries = g_slist_remove(mpq->conf->queries, rd->query);
      /* if any SqlWidDbRel was used for that query, it is removed as well */
      list = mpq->conf->widdbrel_list;
      str = g_strdup_printf("Q%d", rd->query->id);
      while (list) {
	if (!strcmp(((WidRelData *)(list->data))->name, str)) {
	  reldata = (WidRelData *)(list->data);
	  mpq->conf->widdbrel_list = g_slist_remove_link(
					  mpq->conf->widdbrel_list, list);
	  g_free(reldata->name);
	  list = reldata->nodes;
	  while (list) {
	    g_free(list->data);
	    list = g_slist_remove_link(list, list);
	  }
	  g_free(reldata);
	}
	else
	  list = g_slist_next(list);
      }
      g_free(str);
      gtk_object_destroy(GTK_OBJECT(rd->query));
      rd->query = NULL;
    }
    g_free(rd); 
  }


  /* sensitiveness of the buttons */
  if (mpq->sel_row == row) {
    mpq->sel_row = -1;
    gtk_widget_set_sensitive(mpq->remove_query, FALSE);
    gtk_widget_set_sensitive(mpq->edit_query, FALSE);
    gtk_widget_set_sensitive(mpq->exec_query, FALSE);    
  }
}

/*************************************************************************/
/*                                                                       */
/* Action buttons                                                        */
/*                                                                       */
/*************************************************************************/

/***************/
/* ADD A QUERY */
/***************/
/* - if q!=NULL, it will return TRUE if the name is present, but is not the
     one of SqlQuery q, and FALSE otherwise
   - if q==NULL, it will return TRUE if the name qlready exists 
     (in any SqlQuery), and FALSE otherwise 
*/
static gboolean name_already_exists(MainPageQuery *mpq, 
				    SqlQuery *q, gchar *name)
{
  gint i, nb;
  gboolean result = FALSE;
  gchar *str;
  Row_Data *rd;
  SqlQuery *qu;

  nb = GTK_CLIST(mpq->clist)->rows;
  i = 0;
  while (i<nb) {
    rd = (Row_Data *) gtk_clist_get_row_data(GTK_CLIST(mpq->clist), i);
    qu = rd->query; /* qu can be NULL! */
    gtk_clist_get_text(GTK_CLIST(mpq->clist), i, 0, &str);
    if (!strcmp(str, name))
      if ((q && (qu!=q)) || !q)
	result = TRUE;
    i++;
  }

  return result;
}

void main_page_query_add_query(MainPageQuery *mpq, SqlQuery *q)
{
  Row_Data *rd;
  gchar *row[2];
  gint rid;

  rd = (Row_Data *) g_malloc(sizeof(Row_Data));
  rd->query = q;
  rd->edit_dlg = NULL;
  rd->exec_dlg = NULL;
  row[0] = g_strdup(q->name);
  if (q->descr)
    row[1] = g_strdup(q->descr);
  else
    row[1] = g_strdup("");
  rid = gtk_clist_append(GTK_CLIST(mpq->clist), row);
  g_free(row[0]);
  g_free(row[1]);
  gtk_clist_set_row_data(GTK_CLIST(mpq->clist), rid, rd);
  gtk_signal_connect(GTK_OBJECT(rd->query), "name_changed",
		     GTK_SIGNAL_FUNC(edit_query_query_changed_cb), mpq);
}

static void create_dialog_clicked_cb(GnomeDialog * dialog, gint button_number, 
				     gASQL_Main_Config *conf)
{
  GtkWidget *entry;
  gchar *str, *query;
  SqlQuery *q;
  MainPageQuery *mpq;
  gboolean leave = TRUE;

  mpq = gtk_object_get_data(GTK_OBJECT(dialog), "mpq");
  entry = GTK_WIDGET(gtk_object_get_user_data(GTK_OBJECT(dialog)));

  switch (button_number) {
  case 0: /* OK button */
    /* is the name OK (not already used) ? */
    if (name_already_exists(mpq, NULL, gtk_entry_get_text(GTK_ENTRY(entry)))) {
      leave = FALSE;
      query = g_strdup_printf(_("A query named \"%s\" already exists,\n"
			      "choose another name"), 
			      gtk_entry_get_text(GTK_ENTRY(entry)));
      gnome_app_error(GNOME_APP(mpq->conf->app), query);
      g_free(query);
    }
    else {
      /* new structure */
      str = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
      q = SQL_QUERY(sql_query_new(str, conf));/* automatically add query to the
						 conf->queries list */
      main_page_query_add_query(mpq, q);
      g_free(str);
      break;
    }
  case 1: /* Cancel Button */
    break;
  default:
  };

  if (leave) {
    gnome_dialog_close(dialog);
    mpq->new_dlg = NULL;
  }
}

/* sets the sensitiveness of the OK button in the dialog */
static void create_dialog_entry_changed_cb(GtkWidget *wid, gpointer data)
{
  GnomeDialog *dlg;
  gchar *str;
  GtkWidget *entry;

  dlg = GNOME_DIALOG(data);
  entry = GTK_WIDGET(gtk_object_get_user_data(GTK_OBJECT(data)));
  str = gtk_entry_get_text(GTK_ENTRY(entry));
  if (str && (*str!='\0'))
    gnome_dialog_set_sensitive(GNOME_DIALOG(dlg), 0, TRUE);
  else
    gnome_dialog_set_sensitive(GNOME_DIALOG(dlg), 0, FALSE);
}

static void create_query_button_cb(GtkWidget *button, gpointer data)
{
  MainPageQuery *mpq;
  GtkWidget *wid, *dlg;

  mpq = MAIN_PAGE_QUERY(data);
  if (!mpq->new_dlg) {
    dlg = gnome_dialog_new(_("Creation of a new query"), 
			   GNOME_STOCK_BUTTON_OK,
			   GNOME_STOCK_BUTTON_CANCEL, NULL); 
    
    wid = gtk_label_new(_("Query Name:"));
    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dlg)->vbox),
		       wid, TRUE, TRUE, GNOME_PAD);
    gtk_widget_show(wid);
    
    wid = gtk_entry_new();
    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dlg)->vbox),
		       wid, TRUE, TRUE, GNOME_PAD);
    gtk_widget_show(wid);
    gtk_object_set_user_data(GTK_OBJECT(dlg), wid);
    gtk_object_set_data(GTK_OBJECT(dlg), "mpq", mpq);
    gtk_signal_connect(GTK_OBJECT(wid), "changed",
		       GTK_SIGNAL_FUNC(create_dialog_entry_changed_cb), dlg);
    
    gtk_signal_connect(GTK_OBJECT(dlg), "clicked",
		       GTK_SIGNAL_FUNC(create_dialog_clicked_cb),
		       mpq->conf);
    
    gnome_dialog_set_sensitive(GNOME_DIALOG(dlg), 0, FALSE);
    mpq->new_dlg = dlg;
    gtk_widget_show(dlg);
  }
}

/******************/
/* REMOVE A QUERY */
/******************/
static void drop_query_question_cb(gint reply, gpointer data)
{
  MainPageQuery *mpq;

  mpq = MAIN_PAGE_QUERY(data);
  if (reply == GNOME_YES) 
    clist_remove_a_query(mpq, mpq->sel_row);
}

static void remove_query_dlg_destroy(GtkObject *obj, gpointer data)
{
  gnome_dialog_close(GNOME_DIALOG(data));
}

static void drop_query_button_cb(GtkWidget *button, gpointer data)
{
  MainPageQuery *mpq;
  GtkWidget *dlg;
  gchar *str, *que;

  mpq = MAIN_PAGE_QUERY(data);
  if (mpq->sel_row != -1) {
    gtk_clist_get_text(GTK_CLIST(mpq->clist), 
		       mpq->sel_row, 0, &str);
    que = g_strdup_printf(_("Remove the query %s?"), str);
    dlg = gnome_app_question_modal(GNOME_APP(mpq->conf->app), que, 
				   (GnomeReplyCallback) drop_query_question_cb,
				   mpq);
    gtk_signal_connect_while_alive(GTK_OBJECT(mpq->conf->srv), 
				   "conn_closed",
				   GTK_SIGNAL_FUNC(remove_query_dlg_destroy),
				   dlg, GTK_OBJECT(dlg));
    g_free(que);
    gtk_widget_show(dlg);
  }
}


/******************/
/* EDIT   A QUERY */
/******************/
static void edit_query_query_changed_cb(SqlQuery *q, gpointer data)
{
  MainPageQuery *mpq;
  gint nb, i;
  gboolean found;
  Row_Data *rd;

  mpq = MAIN_PAGE_QUERY(data);
  found = FALSE;
  nb = GTK_CLIST(mpq->clist)->rows;
  i=0;
  while ((i<nb) && !found) {
    rd = gtk_clist_get_row_data(GTK_CLIST(mpq->clist), i);
    if (rd && (rd->query == q)) {
      found = TRUE;
      /* updating names and description */
      if (name_already_exists(mpq, q, q->name)) { /* there is a duplicate */
	gchar *str = NULL;
	gint i = 0;

	do {
	  if (str)
	    g_free(str);
	  str = g_strdup_printf("%s_%d", q->name, i);
	  i++;
	} while (name_already_exists(mpq, NULL, str));
	g_free(q->name);
	q->name = str;
      }
      if (*(q->name) == '\0') { /* name = '\0' */
	gchar *str = NULL;
	gint i = 0;

	do {
	  if (str)
	    g_free(str);
	  str = g_strdup_printf(_("Query_%d"), i);
	  i++;
	} while (name_already_exists(mpq, NULL, str));
	g_free(q->name);
	q->name = str;
      }
      gtk_clist_set_text(GTK_CLIST(mpq->clist), i, 0, q->name);
      if (q->descr)
	gtk_clist_set_text(GTK_CLIST(mpq->clist), i, 1, q->descr);
      else
	gtk_clist_set_text(GTK_CLIST(mpq->clist), i, 1, "");
    }
    i++;
  }
}

static void query_edit_dlg_destroy_cb(GtkObject *obj, Row_Data *rd)
{
  rd->edit_dlg = NULL;
}

static void edit_query_button_cb(GtkWidget *button, gpointer data)
{
  MainPageQuery *mpq;
  Row_Data *rd;
  GtkWidget *dlg;

  mpq = MAIN_PAGE_QUERY(data);
  if (mpq->sel_row != -1) {
    rd = (Row_Data *) gtk_clist_get_row_data(GTK_CLIST(mpq->clist), 
					     mpq->sel_row);
    if (!rd->edit_dlg) {
      /* when the DLG is closed, query_edit_dlg_destroy_cb is called */
      dlg = sql_query_edit_new_dlg(rd->query);
      gtk_signal_connect(GTK_OBJECT(dlg), "destroy",
			 GTK_SIGNAL_FUNC(query_edit_dlg_destroy_cb), 
			 rd);
      rd->edit_dlg = dlg;
      gtk_widget_show(dlg);
    }
    else 
      gdk_window_raise(rd->edit_dlg->window);
  }
}

/******************/
/* EXEC   A QUERY */
/******************/

static void query_exec_dlg_destroy_cb(GtkObject *obj, Row_Data *rd)
{
  rd->exec_dlg = NULL;
}

static void exec_query_button_cb(GtkWidget *button, gpointer data)
{
  MainPageQuery *mpq;
  Row_Data *rd;
  gchar *name, *descr;
  GtkWidget *dlg;

  mpq = MAIN_PAGE_QUERY(data);
  if (mpq->sel_row != -1) {
    rd = gtk_clist_get_row_data(GTK_CLIST(mpq->clist), mpq->sel_row);
    if (!rd->exec_dlg) {
      if (!rd->query) {
	gtk_clist_get_text(GTK_CLIST(mpq->clist), 
			   mpq->sel_row, 0, &name);
	rd->query = SQL_QUERY(sql_query_new(name, mpq->conf));
	gtk_clist_get_text(GTK_CLIST(mpq->clist), 
			   mpq->sel_row, 1, &descr);
      }

      /* when the DLG is closed, query_edit_dlg_destroy_cb is called */
      dlg = sql_query_exec_new_dlg(rd->query);
      gtk_signal_connect(GTK_OBJECT(dlg), "destroy",
			 GTK_SIGNAL_FUNC(query_exec_dlg_destroy_cb), 
			 rd);
      rd->exec_dlg = dlg;
      gtk_widget_show(dlg);
    }
    else 
      gdk_window_raise(rd->exec_dlg->window);
  }
}
